@@ -1,13 +1,12 @@
-"""Constants like http status code, log levels and method types."""
+"""Constants like http status code and method types."""
+
+from operator import itemgetter
 
 __author__ = "Ondrej Tuma (McBig) <mcbig@zeropage.cz>"
-__date__ = "11 Nov 2015"
-__version__ = "1.6.1"     # https://www.python.org/dev/peps/pep-0386/
+__date__ = "19 Feb 2021"
+__version__ = "2.3.2"       # https://www.python.org/dev/peps/pep-0386/
 
-OK = 0
-DONE = 1
-DECLINED = 2
-HTTP_ERROR = 3
+DECLINED = 0
 
 HTTP_CONTINUE = 100
 HTTP_SWITCHING_PROTOCOLS = 101
@@ -45,6 +44,7 @@ HTTP_REQUEST_URI_TOO_LARGE = 414
 HTTP_UNSUPPORTED_MEDIA_TYPE = 415
 HTTP_RANGE_NOT_SATISFIABLE = 416
 HTTP_EXPECTATION_FAILED = 417
+HTTP_I_AM_A_TEAPOT = 418
 HTTP_UNPROCESSABLE_ENTITY = 422
 HTTP_LOCKED = 423
 HTTP_FAILED_DEPENDENCY = 424
@@ -58,26 +58,6 @@ HTTP_VARIANT_ALSO_VARIES = 506
 HTTP_INSUFFICIENT_STORAGE = 507
 HTTP_NOT_EXTENDED = 510
 
-LOG_EMERG = (0, 'S')
-LOG_ALERT = (1, 'A')
-LOG_CRIT = (2, 'C')
-LOG_ERR = (3, 'E')
-LOG_WARNING = (4, 'W')
-LOG_NOTICE = (5, 'N')
-LOG_INFO = (6, 'I')
-LOG_DEBUG = (7, 'D')
-LOG_NOERRNO = (8, '0')
-
-# log levels which could be set with poor_LogLevel environment variable
-levels = {'emerg':  LOG_EMERG,
-          'alert':  LOG_ALERT,
-          'crit':   LOG_CRIT,
-          'error':  LOG_ERR,
-          'warn':   LOG_WARNING,
-          'notice': LOG_NOTICE,
-          'info':   LOG_INFO,
-          'debug':  LOG_DEBUG}
-
 METHOD_HEAD = 1
 METHOD_GET = 2
 METHOD_POST = 4
@@ -104,5 +84,4 @@ methods = {'HEAD': METHOD_HEAD,
            'CONNECT': METHOD_CONNECT,
            'PATCH': METHOD_PATCH}
 
-from operator import itemgetter
 sorted_methods = sorted(methods.items(), key=itemgetter(1))
